@@ -135,12 +135,13 @@ def generate_pie_style(widget):
 	# * gtk.STATE_SELECTED - The state of a widget that is selected e.g. selected text in a gtk.Entry widget
 	# * gtk.STATE_INSENSITIVE - The state of a widget that is insensitive and will not respond to any events e.g. cannot be activated, selected or prelit.
 
+	widget.ensure_style()
 	gtkStyle = widget.get_style()
 	sliceStyle = dict(
 		(gtkStyleState, {
 			"text": convert_color(gtkStyle.text[gtkStyleState]),
 			"fill": convert_color(gtkStyle.bg[gtkStyleState]),
-			"stroke": convert_color(gtkStyle.bg[gtkStyleState]),
+			"stroke": None,
 		})
 		for gtkStyleState in (
 			gtk.STATE_NORMAL, gtk.STATE_ACTIVE, gtk.STATE_PRELIGHT, gtk.STATE_SELECTED, gtk.STATE_INSENSITIVE
@@ -221,33 +222,20 @@ class PieSlice(object):
 		outerRadius = self._pie().outerRadius
 
 		fillColor = self._style[styleState]["fill"]
-		strokeColor = self._style[styleState]["stroke"]
+		if not fillColor:
+			return
+
 		if self._direction == self.SLICE_CENTER:
-			if fillColor:
-				context.arc(
-					centerPosition[0],
-					centerPosition[1],
-					radius,
-					0,
-					2 * math.pi
-				)
+			context.arc(
+				centerPosition[0],
+				centerPosition[1],
+				radius,
+				0,
+				2 * math.pi
+			)
 
-				context.set_source_rgb(*fillColor)
-				if strokeColor:
-					context.fill_preserve()
-				else:
-					context.fill()
-
-			if strokeColor:
-				context.arc(
-					centerPosition[0],
-					centerPosition[1],
-					radius,
-					0,
-					2 * math.pi
-				)
-				context.set_source_rgb(*strokeColor)
-				context.stroke()
+			context.set_source_rgb(*fillColor)
+			context.fill()
 		else:
 			sliceCenterAngle = self.quadrant_to_theta(self._direction)
 			sliceArcWidth = 2*math.pi / self.MAX_ANGULAR_SLICES
@@ -270,16 +258,8 @@ class PieSlice(object):
 			)
 			context.close_path()
 
-			if fillColor:
-				context.set_source_rgb(*fillColor)
-				if strokeColor:
-					context.fill_preserve()
-				else:
-					context.fill()
-
-			if strokeColor:
-				context.set_source_rgb(*strokeColor)
-				context.stroke()
+			context.set_source_rgb(*fillColor)
+			context.fill()
 
 	def activate(self):
 		self._handler(self._pie(), self, self._direction)
@@ -505,63 +485,40 @@ class PieMenu(gtk.DrawingArea):
 	def __generate_draw_event(self):
 		if self.window is None:
 			return
-		rect = self.get_allocation()
-		self.window.invalidate_rect(rect, True)
+		self.queue_draw()
 
 	def _on_expose(self, widget, event):
-		# @bug Not getting all of the invalidation events needed on Hildon
-		# @bug Not highlighting proper slice besides center on Hildon
 		cairoContext = self.window.cairo_create()
 		pangoContext = self.create_pango_context()
 		textLayout = pango.Layout(pangoContext)
 
 		rect = self.get_allocation()
+		position = 0, 0
+		dimensions = rect.width, rect.height
 
-		self.centerPosition = event.area.x + event.area.width / 2, event.area.y + event.area.height / 2
-		self.radius = max(rect.width, rect.width) / 3 / 2
-		self.outerRadius = max(rect.width, rect.height)
-
-		for slice in self.__slices.itervalues():
-			isSelected = (slice is self.__activeSlice)
-			if not isSelected:
-				cairoContext.rectangle(
-					event.area.x,
-					event.area.y,
-					event.area.width,
-					event.area.height,
-				)
-				cairoContext.clip()
-				slice.draw_bg(self.__styleState, isSelected, cairoContext, textLayout)
+		self.centerPosition = position[0] + dimensions[0] / 2, position[1] + dimensions[1] / 2
+		self.outerRadius = max(*dimensions) # be larger than the view
+		self.radius = self.outerRadius / (3*2) # fit inside the middle cell
 
+		# Draw Background
 		cairoContext.rectangle(
-			event.area.x,
-			event.area.y,
-			event.area.width,
-			event.area.height,
+			position[0],
+			position[1],
+			dimensions[0],
+			dimensions[1],
 		)
-		cairoContext.clip()
+		cairoContext.set_source_rgb(*self.sliceStyle[self.__styleState]["fill"])
+		cairoContext.fill()
+
 		isSelected = self.__clickPosition != (0, 0)
 		self.__activeSlice.draw_bg(self.__styleState, isSelected, cairoContext, textLayout)
 
+		# Draw Foreground
 		for slice in self.__slices.itervalues():
 			isSelected = (slice is self.__activeSlice)
 			if not isSelected:
-				cairoContext.rectangle(
-					event.area.x,
-					event.area.y,
-					event.area.width,
-					event.area.height,
-				)
-				cairoContext.clip()
 				slice.draw_fg(self.__styleState, isSelected, cairoContext, textLayout)
 
-		cairoContext.rectangle(
-			event.area.x,
-			event.area.y,
-			event.area.width,
-			event.area.height,
-		)
-		cairoContext.clip()
 		isSelected = self.__clickPosition != (0, 0)
 		self.__activeSlice.draw_fg(self.__styleState, isSelected, cairoContext, textLayout)
 
@@ -639,28 +596,34 @@ class PiePopup(gtk.DrawingArea):
 			gtk.gdk.LEAVE_NOTIFY_MASK
 		)
 
+		self.__popped = False
+		self.__styleState = gtk.STATE_NORMAL
 		self.__activeSlice = None
 		self.__slices = {}
 		self.__localSlices = {}
-		for direction in PieSlice.SLICE_DIRECTIONS:
-			self.add_slice(NullPieSlice(), direction)
-		#self.__activeSlice = NullPieSlice()
-		#self.__activeSlice.menu_init(self, PieSlice.SLICE_CENTER)
 
 		self.__clickPosition = 0, 0
-		self.__popupWindow = None
-		self.__pie = None
 		self.__popupTimeDelay = None
-		self.__styleState = gtk.STATE_NORMAL
+
+		self.__pie = None
+		self.__pie = PieMenu(self.sliceStyle)
+		self.__pie.connect("button_release_event", self._on_button_release)
+		self.__pie.show()
+
+		self.__popupWindow = gtk.Window(type = gtk.WINDOW_POPUP)
+		self.__popupWindow.set_title("")
+		self.__popupWindow.add(self.__pie)
+
+		self.add_slice(NullPieSlice(), PieSlice.SLICE_CENTER)
 
 	def add_slice(self, slice, direction):
 		assert direction in PieSlice.SLICE_DIRECTIONS
 
-		self.__slices[direction] = slice
-		if self.showAllSlices or direction == PieSlice.SLICE_CENTER:
-			self.__localSlices[direction] = copy.copy(slice)
-			self.__localSlices[direction].menu_init(self, direction)
+		self.__pie.add_slice(copy.copy(slice), direction)
 
+		if self.showAllSlices or direction == PieSlice.SLICE_CENTER:
+			self.__localSlices[direction] = slice
+			self.__localSlices[direction].menu_init(self, direction)
 		if direction == PieSlice.SLICE_CENTER:
 			self.__activeSlice = self.__localSlices[PieSlice.SLICE_CENTER]
 
@@ -683,63 +646,37 @@ class PiePopup(gtk.DrawingArea):
 			self.__generate_draw_event()
 
 	def __generate_draw_event(self):
-		rect = self.get_allocation()
-		rect.x = 0
-		rect.y = 0
-		self.window.invalidate_rect(rect, True)
+		self.queue_draw()
 
 	def _on_expose(self, widget, event):
+		rect = self.get_allocation()
+		position = 0, 0
+		dimensions = rect.width, rect.height
+
+		# update sizing information
+		self.centerPosition = position[0] + dimensions[0] / 2, position[1] + dimensions[1] / 2
+		self.outerRadius = max(*dimensions) # be larger than the view
+		self.radius = self.outerRadius / (3*2) # fit inside the middle cell
+
+		# Draw Background
 		cairoContext = self.window.cairo_create()
+		cairoContext.rectangle(
+			position[0],
+			position[1],
+			dimensions[0],
+			dimensions[1],
+		)
+		cairoContext.set_source_rgb(*self.sliceStyle[self.__styleState]["fill"])
+		cairoContext.fill()
+
+		# Draw Foreground
 		pangoContext = self.create_pango_context()
 		textLayout = pango.Layout(pangoContext)
-
-		rect = self.get_allocation()
-
-		self.centerPosition = event.area.x + event.area.width / 2, event.area.y + event.area.height / 2
-		self.radius = max(rect.width, rect.width) / 3 / 2
-		self.outerRadius = max(rect.width, rect.height)
-
 		for slice in self.__localSlices.itervalues():
 			isSelected = (slice is self.__activeSlice)
 			if not isSelected:
-				cairoContext.rectangle(
-					event.area.x,
-					event.area.y,
-					event.area.width,
-					event.area.height,
-				)
-				cairoContext.clip()
-				slice.draw_bg(self.__styleState, isSelected, cairoContext, textLayout)
-
-		cairoContext.rectangle(
-			event.area.x,
-			event.area.y,
-			event.area.width,
-			event.area.height,
-		)
-		cairoContext.clip()
-		isSelected = self.__clickPosition != (0, 0)
-		self.__activeSlice.draw_bg(self.__styleState, isSelected, cairoContext, textLayout)
-
-		for slice in self.__localSlices.itervalues():
-			isSelected = (slice is self.__activeSlice)
-			if not isSelected:
-				cairoContext.rectangle(
-					event.area.x,
-					event.area.y,
-					event.area.width,
-					event.area.height,
-				)
-				cairoContext.clip()
 				slice.draw_fg(self.__styleState, isSelected, cairoContext, textLayout)
 
-		cairoContext.rectangle(
-			event.area.x,
-			event.area.y,
-			event.area.width,
-			event.area.height,
-		)
-		cairoContext.clip()
 		isSelected = self.__clickPosition != (0, 0)
 		self.__activeSlice.draw_fg(self.__styleState, isSelected, cairoContext, textLayout)
 
@@ -753,7 +690,7 @@ class PiePopup(gtk.DrawingArea):
 
 	def _on_motion_notify(self, widget, event):
 		self.__update_state(event.get_coords())
-		if self.__popupWindow is None or self.__pie is None:
+		if not self.__popped:
 			return
 
 		mousePosition = event.get_root_coords()
@@ -763,12 +700,17 @@ class PiePopup(gtk.DrawingArea):
 		self.__pie._on_motion_notify(self.__pie, event)
 
 	def _on_button_press(self, widget, event):
-		if len(self.__slices) == 0:
+		if len(self.__localSlices) == 0:
+			return
+
+		if self.__popupTimeDelay is not None:
+			# This press is for a double click for which we do not get a release
+			# self._on_button_release(widget, event)
 			return
 
 		self.__clickPosition = event.get_root_coords()
 		self.__generate_draw_event()
-		self.__popupTimeDelay = gobject.timeout_add(100, self._on_delayed_popup)
+		self.__popupTimeDelay = gobject.timeout_add(200, self._on_delayed_popup)
 
 	def _on_delayed_popup(self):
 		self.__popup(self.__clickPosition)
@@ -777,7 +719,7 @@ class PiePopup(gtk.DrawingArea):
 		return False
 
 	def _on_button_release(self, widget, event):
-		if len(self.__slices) == 0:
+		if len(self.__localSlices) == 0:
 			return
 
 		if self.__popupTimeDelay is None:
@@ -798,54 +740,28 @@ class PiePopup(gtk.DrawingArea):
 		self.__generate_draw_event()
 
 	def __popup(self, position):
-		# @bug Figure out what to do with this assert
-		assert self.__popupWindow is None and self.__pie is None
-		self.__popupWindow = gtk.Window(type = gtk.WINDOW_POPUP)
-		self.__popupWindow.set_title("")
+		assert not self.__popped
+		self.__popped = True
 
 		width, height = 256, 256
 		popupX, popupY = position[0] - width/2, position[1] - height/2
+
 		self.__popupWindow.move(int(popupX), int(popupY))
 		self.__popupWindow.resize(width, height)
-
-		self.__pie = PieMenu(self.sliceStyle)
-		self.__popupWindow.add(self.__pie)
-		for direction, slice in self.__slices.iteritems():
-			self.__pie.add_slice(copy.copy(slice), direction)
 		pieClick = FakeEvent(width/2, height/2, False)
 		self.__pie._on_button_press(self.__pie, pieClick)
-		self.__pie.connect("button_release_event", self._on_button_release)
-
-		#self.__pie.grab_add()
 		self.__pie.grab_focus()
 
-		#gtk.gdk.pointer_grab(
-		#	self.__pie.window,
-		#	True,
-		#	gtk.gdk.BUTTON_PRESS_MASK |
-		#	gtk.gdk.BUTTON_RELEASE_MASK |
-		#	gtk.gdk.ENTER_NOTIFY_MASK |
-		#	gtk.gdk.LEAVE_NOTIFY_MASK |
-		#	gtk.gdk.POINTER_MOTION_MASK
-		#)
-
-		#gtk.gdk.keyboard_grab(
-		#	self.__pie.window,
-		#	owner_events=True
-		#)
-
-		self.__popupWindow.show_all()
+		self.__popupWindow.show()
 
 	def __unpop(self):
+		assert self.__popped
+		self.__popped = False
+
 		piePosition = self.__popupWindow.get_position()
-
-		#self.__pie.grab_remove()
 		self.grab_focus()
-		#gtk.gdk.pointer_ungrab()
 
-		self.__popupWindow.destroy()
-		self.__popupWindow = None
-		self.__pie = None
+		self.__popupWindow.hide()
 
 
 gobject.type_register(PiePopup)
